# -*- coding: utf-8 -*-
"""
Created on Mon Jul 20 14:01:02 2020

@author: Ysc
"""
# utilsimagedata.py
import os
import tensorflow as tf
from keras.models import Sequential 
from keras.layers import Dense, Dropout, Flatten
from keras.constraints import maxnorm
from keras.optimizers import Adam
from keras.layers.convolutional import Conv2D
from keras.layers.convolutional import MaxPooling2D
from keras.utils import np_utils
import numpy as np
import time
import matplotlib.pyplot as plt


data_root = ("D:/아두이노자료_17_03_24/인공지능응용공학/ImageClassfication/DataSet")
img_height = 224
img_width = 224
batch_size = 15
data_dir = str(data_root)


train_ds = tf.keras.utils.image_dataset_from_directory(
  data_dir,
  validation_split=0.2,
  subset="training",
  shuffle=True,
  seed=1234,
  image_size=(img_height, img_width),
  batch_size=batch_size)

val_ds = tf.keras.utils.image_dataset_from_directory(
  data_dir,
  validation_split=0.2,
  subset="validation",
  shuffle=True,
  seed=1234,
  image_size=(img_height, img_width),
  batch_size=batch_size)


class_names = train_ds.class_names
print(class_names)


for image_batch, label_batch in train_ds:
    print(image_batch.shape)
    print(label_batch.shape)
    break

train_image_batch = image_batch
train_label_batch = label_batch

for valid_image_batch, valid_label_batch in val_ds:
    print(valid_image_batch.shape)
    print(valid_label_batch.shape)
    break


normalization_layer = tf.keras.layers.Rescaling(1./255)

normalized_ds = train_ds.map(lambda x, y: (normalization_layer(x), y))
image_batch, labels_batch = next(iter(normalized_ds))
first_image = image_batch[0]
# Notice the pixel values are now in `[0,1]`.
print(np.min(first_image), np.max(first_image))

plt.figure()
plt.imshow(image_batch[2])
plt.show()

num_classes = 6
def create_model(num_classes):    
    model = tf.keras.Sequential([
        tf.keras.layers.Rescaling(1./255),
        tf.keras.layers.Conv2D(32, 3, activation='relu'),
        tf.keras.layers.MaxPooling2D(),
        tf.keras.layers.Conv2D(32, 3, activation='relu'),
        tf.keras.layers.MaxPooling2D(),
        tf.keras.layers.Conv2D(32, 3, activation='relu'),
        tf.keras.layers.MaxPooling2D(),
        tf.keras.layers.Flatten(),
        tf.keras.layers.Dense(128, activation='relu'),
        tf.keras.layers.Dense(num_classes, activation='softmax')
        ])
    
# callback 파라메터를 어떻게 심을 것인지 알아보자.
    model.compile(
        optimizer='adam',
        loss=tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True),
        metrics=['accuracy'])
    
    return model

model = create_model(num_classes)


checkpoint_path = "training_f/cp.ckpt"
checkpoint_dir = os.path.dirname(checkpoint_path)
print('dir name : ', checkpoint_dir)
# Create a callback that saves the model's weights
cp_callback = tf.keras.callbacks.ModelCheckpoint(filepath=checkpoint_path,
save_weights_only=True,
verbose=1)

start = time.time()

history = model.fit(
  train_ds,
  validation_data=val_ds,
  epochs=20,
  callbacks=[cp_callback]
)
end = time.time()
print('train time', end-start, '초')

model = create_model(num_classes)
# Loads the weights
model.load_weights(checkpoint_path)

loss, acc = model.evaluate(valid_image_batch, valid_label_batch, verbose=2)
print("Untrained model, accuracy: {:5.2f}%".format(100 * acc))

loss, acc = model.evaluate(train_image_batch, train_label_batch, verbose=2)
print("Already trained model, accuracy: {:5.2f}%".format(100 * acc))

"""
샘플결과
5/5 [==============================] - 3s 637ms/step - loss: 0.0013 - accuracy: 1.0000 - val_loss: 1.5013 - val_accuracy: 0.7222
train time 70.09474420547485 초
1/1 - 0s - loss: 1.5335 - accuracy: 0.7333 - 380ms/epoch - 380ms/step
Untrained model, accuracy: 73.33%
1/1 - 0s - loss: 0.0011 - accuracy: 1.0000 - 110ms/epoch - 110ms/step
Already trained model, accuracy: 100.00%
"""




